var app = angular.module('app', ['ionic', 'ngStorage', 'pascalprecht.translate', 'app.constants', 'app.changelog', 'app.filters', 'app.controllers', 'app.routes', 'app.services', 'app.directives', 'ngResource', 'ngCordova', 'tmh.dynamicLocale', 'ridge-speedometer'])

.config(function(tmhDynamicLocaleProvider) {
    tmhDynamicLocaleProvider.localeLocationPattern('lib/angular-i18n/angular-locale_{{locale}}.js');
})

.config(function($translateProvider, AppConfig, $localStorageProvider) {
    var lang = $localStorageProvider.get('lang');
    if (!lang) {
        $localStorageProvider.set("lang", AppConfig.lang);
    }
    $translateProvider.useSanitizeValueStrategy();
    $translateProvider.useStaticFilesLoader({
        prefix: "locale-",
        suffix: ".json"
    });

    $translateProvider.preferredLanguage(AppConfig.lang);
    $translateProvider.fallbackLanguage(AppConfig.lang);
})

.run(function($state, Toast, Speedometer, Timer, $rootScope, $ionicHistory, $ionicPopup, FrontendService, $ionicLoading, $ionicPlatform, AppConfig, ConnectivityMonitor, $http, $localStorage, $translate, ServerConnection, $ionicSideMenuDelegate, Setting, $cordovaGeolocation, GetMapDataExService, $timeout) {

    if (!AppConfig.hasOwnProperty("navigation") || !AppConfig.navigation.hasOwnProperty("AllowNavigation")) {
        AppConfig.navigation = {
            AllowNavigation: false,
            UrbanCycles: false
        };
    }

    $rootScope.ebikeRange = function(battery) {
        return Math.floor(battery * 0.8);
    }

    $rootScope.keys = Object.keys;
    $rootScope.map = false;
    $rootScope.ebikeTraceShow = false;
    $rootScope.UserLocation = false;
    $rootScope.UserSpeedometer = {
        speed: 0,
        altitude: "?"
    };
    $rootScope.UserLocationLoading = false;
    $rootScope.GetMapDataEx = {
        LocationID: false,
        ReturnBikeExternal: false,
        ReturnBikeExternalFree: false
    };
    $rootScope.preventAutoLogin = false;

    var allowedModules = [{
        key: "OngoingRentals",
        priority: 1,
        show: true,
        element: "ongoing-rentals"
    }, {
        key: "NearestStation",
        priority: 2,
        show: true,
        element: "nearest-station"
    }, {
        key: "GetMapDataEx",
        priority: 3,
        show: true,
        element: "list-get-map-data-ex"
    }, {
        key: "GetCustomerCards",
        priority: 4,
        show: true,
        element: "get-customer-cards"
    }, {
        key: "GetCustomerContracts",
        priority: 5,
        show: true,
        element: "get-customer-contracts"
    }];

    if (AppConfig.bikes.ebikes === true) {
        allowedModules.push({
            key: "EBikeMotorProfiles",
            priority: allowedModules.length + 1,
            show: true,
            element: "ebike-motor-profiles"
        });
    } else {
        if ($localStorage.dashboardModules) {
            for (var i in $localStorage.dashboardModules) {
                if ($localStorage.dashboardModules[i].key == "EBikeMotorProfiles") {
                    $localStorage.dashboardModules.splice(i, 1);
                }
            }
        }
    }

    if (!angular.isArray($localStorage.dashboardModules)) {
        var backupLogin = angular.copy($localStorage.Login);
        $localStorage.$reset();
        $localStorage.Login = backupLogin;
    }

    $rootScope.defaultLocalStorage = {
        Advert: {},
        CacheSettings: {
            GetCustomerRentalsOngoing: 10000
        },
        Cache: {},
        Notifications: {},
        BackendSettings: {},
        OAuth: false,
        lang: AppConfig.lang,
        anonymous: false,
        NavigationApp: "NativeApp",
        intro: false,
        LastChangelog: false,
        distanceUnits: "km",
        speedometer: "digital",
        landingPage: "menu.dashboard",
        dashboardModules: allowedModules,
        PreAuth: [],
        GetBusinessRules: [],
        BugReport: {
            Allowed: false,
            EmailAddress: []
        },
        GetMapDataEx: {
            mapData: false,
            routes: false
        },
        Gateways: [],
        GetCustomerRentals: {
            ongoing: {},
            closed: {}
        },
        News: {
            News: {}
        },
        Faqs: {
            News: {}
        },
        Contacts: {
            News: {}
        },
        Infos: {
            News: {}
        },
        spinner: {},
        temp: {
            RentReturnList: "rent"
        }
    };

    if (AppConfig.navigation.UrbanCycles === true) {
        $rootScope.defaultLocalStorage.NavigationApp = "UrbanCycles";
    }

    $localStorage.$default($rootScope.defaultLocalStorage);

    AppConfig.BackendSettings = $localStorage.BackendSettings;

    delete $localStorage.spinner.GetCustomerContracts;
    delete $localStorage.spinner.GetCustomerRentalsOngoing;
    delete $localStorage.spinner.GetCustomerRentalsClosed;
    delete $localStorage.spinner.GetMapDataEx;
    delete $localStorage.spinner.GetCustomerCards;
    delete $localStorage.spinner.EBikeMotorProfiles;

    if ($localStorage.lang == "ar") {
        $rootScope.dir = "rtl";
        $rootScope.iconPosition = 'item-icon-right';
    } else {
        $rootScope.dir = "ltr";
        $rootScope.iconPosition = 'item-icon-left';
    }

    $rootScope.$on('$stateChangeStart',
        function(event, toState, toParams, fromState, fromParams) {
            if (toState.name == "menu.GetMapDataEx") {
                if ($rootScope.map !== false) {
                    $rootScope.map.setClickable(true);
                }
            } else {
                if ($rootScope.map !== false) {
                    $rootScope.map.setClickable(false);
                }
            }
            if (fromState.name == "menu.GetMapDataEx") {
                if ($rootScope.map !== false) {
                    $rootScope.map.setClickable(false);
                }
            }
        }
    );

    $translate.use($localStorage.lang);
    if ($localStorage.OAuth && $localStorage.OAuth !== null && $localStorage.OAuth.hasOwnProperty("access_token") == true) {
        $http.defaults.headers.common['Authorization'] = 'Bearer ' + $localStorage.OAuth.access_token;
    }

    $rootScope.AppConfig = AppConfig;

    $rootScope.initAppData = function(attempt) {
        $localStorage.TKAccountActivities = false;
        AppConfig.AddCredit.gateways = $localStorage.Gateways;
        AppConfig.BackendSettings.RENTAL_ALLOW_EXTERNAL_RETURN_ANYWHERE = 0;
        attempt = attempt || 1;
        Setting.query(function(data) {
            $localStorage.BackendSettings = data;
            AppConfig.BackendSettings = data;
            FrontendService.get("gateway", false, function(data) {
                $localStorage.Gateways = data;
                AppConfig.AddCredit.gateways = data;
            });
            FrontendService.get("setting", false, function(data) {
                $localStorage.FrontendSettings = data;
            });
            GetMapDataExService.get(function() {
                $rootScope.initWatchPosition();
            });
            FrontendService.get("get-business-rules", false, function(data) {
                $localStorage.GetBusinessRules = data.TKBusinessRules;
            });
            FrontendService.get("bug-report", false, function(data) {
                $rootScope.AppConfig.BugReport = data;
            });
        }, function(err) {
            $ionicLoading.show({
                template: '<ion-spinner></ion-spinner>',
                duration: 3000
            });
            $timeout(function() {
                $rootScope.initAppData(attempt + 1);
            }, 3000);
        });
    }

    $ionicPlatform.ready(function() {
        // ionic.Platform.fullScreen();
        if (typeof cordova != "undefined" && cordova.hasOwnProperty("plugins")) {
            // cordova.plugins.notification.local.on("click", function(notification) {
            //     $ionicHistory.nextViewOptions({
            //         disableBack: true
            //     });
            //     $state.go("menu.OngoingRentals");
            // });

            // cordova.plugins.backgroundMode.on('activate', function(x) {
            //     Timer.backgroundMode();
            // });

            // cordova.plugins.backgroundMode.on('deactivate', function(x) {
            //     Timer.foregroundMode();
            // });
        }

        $rootScope.platform = ionic.Platform.platform();
        if ($rootScope.platform == "macintel") $rootScope.platform = "ios";
        if ($rootScope.platform != "ios" && $rootScope.platform != "android" && $rootScope.platform != "macintel") {
            alert("Unsupported or undetected platform, please contact admin@homeport.cz");
            ionic.Platform.exitApp();
        }

        // if (AppConfig.hasOwnProperty("GoogleAnalytics") && $cordovaGoogleAnalytics && ionic.Platform.platform() != "macintel") {
        //     $cordovaGoogleAnalytics.startTrackerWithId(AppConfig.GoogleAnalytics);
        // } else {
        //     AppConfig.GoogleAnalytics = false;
        // }

        if (navigator.hasOwnProperty("app") && navigator.app.hasOwnProperty("overrideButton")) {
            navigator.app.overrideButton("menubutton", true);
            document.addEventListener("menubutton", function() {
                $ionicSideMenuDelegate.toggleLeft();
            }, false);
        }

        $ionicPlatform.registerBackButtonAction(function(event) {
            if ($state.current.name == "login") {
                navigator.app.exitApp();
            } else if ($state.current.name == "menu.bugReport") {
                $rootScope.$broadcast('bugReportBack');
            } else {
                $rootScope.preventAutoLogin = true;
                navigator.app.backHistory();
            }
        }, 100);

        if (ionic.Platform.isWebView()) {
            $rootScope.$on('$cordovaNetwork:offline', function(event, networkState) {
                ConnectivityMonitor.isOffline();
            });
        } else {
            window.addEventListener("offline", function(e) {
                ConnectivityMonitor.isOffline();
            }, false);
        }

        if (window.cordova && window.cordova.plugins && window.cordova.plugins.Keyboard) {
            cordova.plugins.Keyboard.hideKeyboardAccessoryBar(false);
            cordova.plugins.Keyboard.disableScroll(true);
        }
        if (window.StatusBar) {
            StatusBar.styleDefault();
        }

        $rootScope.watchPosition = false;
        $rootScope.UserLocation = false;
        $rootScope.UserMovement = false;
        $rootScope.UserLocationError = false;

        $rootScope.initWatchPosition = function() {
            $rootScope.watchPosition = $cordovaGeolocation.watchPosition(watchOptions = {
                // timeout: 15000,
                enableHighAccuracy: true
            });

            $rootScope.watchPosition.then(
                null,
                function(err) {
                    $rootScope.UserLocationError = err;
                    if ($rootScope.UserLocationError.code != 1 || ($rootScope.UserLocationError.error_code && $rootScope.UserLocationError.error_code != 'service_denied')) {
                        $rootScope.initWatchPosition();
                    } else {
                        $translate('error.geolocation.failed').then(function(string) {
                            Toast.show(string);
                        });
                    }
                },
                function(position) {
                    Speedometer.update(position);
                    position.latLng = {
                        "lat": position.coords.latitude,
                        "lng": position.coords.longitude
                    };

                    $rootScope.UserLocation = position;

                    $rootScope.UserLocationError = false;
                    GetMapDataExService.distance(function() {
                        $rootScope.$broadcast("PositionChanged");
                    });
                });
        }

        $rootScope.initAppData();

    });
});
