angular.module('app.directives', [])

.directive('accountActivity', [function() {
    return {
        restrict: "E",
        replace: true,
        transclude: true,
        templateUrl: 'templates/TKAccountActivity.html'
    };
}])

.directive('rental', [function() {
    return {
        restrict: "E",
        replace: true,
        transclude: true,
        templateUrl: 'templates/TKRental.html'
    };
}])

.directive('headerItem', ['$ionicScrollDelegate', function($ionicScrollDelegate) {
    var directive = {
        restrict: 'C',
        link: function(scope, element, attr) {
            element.bind('click', function() {
                $ionicScrollDelegate.scrollTop(true);
            });
        }
    };

    return directive;
}])

.directive('fullLoader', function() {
    return {
        restrict: 'E',
        scope: {
            icon: '@icon',
            size: '@size',
            text: '@text'
        },
        template: "<div class='fullLoader {{size}}'><span class='spinnerContent'><ion-spinner icon='{{icon}}'></ion-spinner><span class='text' ng-if='text.length > 0' translate='{{text}}'></span></span></div>",
        replace: true
    }
})

.directive('stationDetail', [function() {
    return {
        restrict: 'E',
        templateUrl: "templates/stationDetail.html",
        replace: true,
        controller: function($scope, $state,$rootScope,$window, cordovaLaunchNavigator, BatteryIcon, GetLocation, AppConfig, GetBikesAtLocation, $localStorage, $rootScope, ServerConnection, Toast, $translate, $ionicScrollDelegate, $filter, Timer, GetMapDataExService) {
            $scope.AllowDistance = false;
            $scope.AllowDistancePreAuth = false;
            $scope.BikeRentedPassword = false;
            $scope.messageTooFarFromStation = '';
            $scope.messageTooFarFromStationPreAuth = '';
            $scope.bikeLoading = false;
            $scope.stationData = false;
            $scope.BatteryIcon = BatteryIcon;
            $scope.distance = false;
            $scope.AllowPreAuth = false;


            $scope.$storage = $localStorage;

            $rootScope.$on("GetMapDataExUpdate", function() {
                if ($scope.stationData !== false && $scope.LocationID !== false) {
                    $scope.doRefresh();
                }
            });

            setTimeout(function() {
                $scope.verifyPreAuthContract();
            }, 2000);

            $rootScope.$on("PositionChanged", function() {
                $scope.computeDistance();
            });

            $scope.$on("verifyPreAuth", function() {
                $scope.verifyPreAuth();
            });

            $scope.$on("GetBikesAtLocation", function() {
                $scope.GetBikesAtLocation();
            });

            $scope.$on("RefreshStationDetail", function() {
                $scope.RefreshStationDetail();
            });

            $scope.$on("ActivateFreeBike", function() {
                $scope.ActivateFreeBike();
            });

            $scope.openNavigationApp = function() {
                var destination = [$scope.stationData.Latitude, $scope.stationData.Longitude];
                if ($scope.$storage.NavigationApp == "UrbanCycles") {
                    var deeplink = "https://p29ek.app.goo.gl/?link=http://urbancyclers.cz/plan/%23dlat%3D" + destination[0] + "%26dlon%3D" + destination[1];
                    if ($rootScope.UserLocation !== false) {
                        var start = [$rootScope.UserLocation.latLng.lat, $rootScope.UserLocation.latLng.lng];
                        deeplink = deeplink + "%26olat%3D" + start[0] + "%26olon%3D" + start[1];
                    }

                    deeplink = deeplink + "&apn=com.umotional.bikeapp";
                    window.open(deeplink, "_system");
                } else {
                    cordovaLaunchNavigator.navigate(destination, {}).then(function() {}, function(err) {});
                }
            }

            $scope.verifyPreAuthContract = function() {
                if (!$scope.$storage.TKCustomerContracts) return;
                var ActiveContracts = $filter("filter")($scope.$storage.TKCustomerContracts.Contracts, {
                    StatusCode: 2
                }, true);

                angular.forEach(ActiveContracts, function(data) {
                    if (data.AllowedReservations === true) $scope.AllowPreAuth = true;
                });
            }

            $rootScope.$on("requestFinished", function() {
                $scope.stationData.loading = false;
                $scope.bikeLoading.loading = false;
                $scope.bikeLoading = false;
            });

            $scope.findLocationByID = function(LocationID) {
                var data = $filter("filter")($scope.$storage.GetMapDataEx.mapData.Locations, {
                    LocationID: LocationID
                }, true);
                if (data) {
                    return data[0];
                } else {
                    return false;
                }
            }

            $scope.$watch('$storage.GetMapDataEx.mapData', function() {
                $scope.RefreshStationDetail();
            });

            $scope.ActivateFreeBike = function() {
                $scope.stationData = $scope.findBikeByID($scope.BikeID);
                $scope.stationData.loading = false;
                $scope.BikeRentedPassword = false;
                $scope.$parent.stationData = $scope.stationData;
                $scope.computeDistance();
            }

            $scope.findBikeByID = function(BikeID) {
                if ($scope.$storage.GetMapDataEx.BikesOutOfLocation.length > 0) {
                    var data = $filter("filter")($scope.$storage.GetMapDataEx.BikesOutOfLocation, {
                        BikeID: BikeID
                    }, true);
                    return data[0];
                } else {
                    return false;
                }
            }

            $scope.RefreshStationDetail = function() {
                if ($rootScope.UserLocationError !== false) {
                    $rootScope.initWatchPosition();
                }
                if ($scope.LocationID === false) return;
                $scope.verifyPreAuthContract();
                $scope.stationData = $scope.findLocationByID($scope.LocationID);
                $scope.$parent.stationData = $scope.stationData;
                $scope.$broadcast('scroll.refreshComplete');
                if ($scope.$storage.anonymous == false) {
                    ServerConnection.GetCustomerRentalsServiceOngoing();
                }
                $scope.GetBikesAtLocation();
                $scope.verifyPreAuth();
            }

            $scope.doRefresh = function() {
                if ($scope.stationData !== false && $scope.stationData.hasOwnProperty('LocationID')) {
                    $scope.RefreshStationDetail();
                }
            }

            $scope.RentBikeFree = function(bike) {
                $scope.stationData.loading = true;
                ServerConnection.RentReturnBikeExternalRent({
                    bikeId: bike.BikeID,
                    spikeId: ''
                }, function(data) {
                    ServerConnection.GetCustomerRentalsServiceOngoing(function() {});
                    $translate('pages.GetMapDataEx.RentReturn.RentedBike').then(function(string) {
                        $scope.BikeRentedPassword = string + " " + data.BikePassword;
                        $scope.updateMap();
                    });
                    $scope.stationData.loading = false;
                });
            }


            $scope.RentBike = function(bike) {
                $scope.bikeLoading = bike;
                bike.loading = true;
                ServerConnection.RentReturnBikeExternalRent({
                    bikeId: bike.BikeID,
                    spikeId: ''
                }, function(data) {
                    ServerConnection.GetCustomerRentalsServiceOngoing();
                    $translate('pages.GetMapDataEx.RentReturn.RentedBike').then(function(string) {
                        Toast.show(string + " " + data.BikePassword, "long");
                    });
                    GetBikesAtLocation.query({
                        locationId: $scope.stationData.LocationID
                    }, function(data) {
                        $scope.stationData.AvailableExternalBikes = data;
                    });
                });
            }

            $scope.eBikeDetail = function(ebike) {
                if (ebike.IsEBike) {
                    var batteryStateString = "pages.GetMapDataEx.bikesInStation.batteryStates." + BatteryIcon.info(ebike.Battery, "state");
                    $translate(['pages.GetMapDataEx.bikesInStation.batteryState', batteryStateString]).then(function(string) {
                        Toast.show(string[batteryStateString], "long");
                    });
                }
            }

            $scope.ReturnBike = function(rental) {
                rental.loading = true;
                ServerConnection.RentReturnBikeExternalReturn({
                    locationId: $scope.stationData.LocationID,
                    rentalId: rental.RentalID,
                    spikeId: '',
                    latitude: '',
                    longitude: '',
                    confirmationCode: ''
                }, function(data) {
                    rental.returned = true;
                    rental.loading = false;
                    ServerConnection.GetCustomerRentalsServiceOngoing(function() {
                        if ($scope.$storage.GetCustomerRentals.ongoing.Rentals.length == 0) {
                            $scope.$storage.temp.RentReturnList = "rent";
                        }
                    });
                    GetBikesAtLocation.query({
                        locationId: $scope.stationData.LocationID
                    }, function(data) {
                        $scope.stationData.AvailableExternalBikes = data;
                    });
                    $translate('pages.GetMapDataEx.RentReturn.ReturnedBike').then(function(string) {
                        Toast.show(string);
                    });
                }, function(err) {
                    rental.returned = false;
                    rental.loading = false;
                    Toast.show(err.data.Message);
                });
            }

            $scope.verifyPreAuth = function() {

                Timer.add("PreAuthChecker", false, function($scope) {
                    angular.forEach($scope.$storage.PreAuth, function(bike, key) {
                        ServerConnection.PreAuthDetail({
                            reservationId: bike.PreAuth.Id
                        }, function(reservation) {
                            if (reservation.StateCode == 2) {
                                $translate('pages.GetMapDataEx.PreAuth.2').then(function(string) {
                                    Toast.show(string, 'long');
                                });
                            }
                            if (reservation.StateCode != 1) {
                                $scope.DeletePreAuth(bike.PreAuth.Id);
                                $scope.doRefresh();
                            }
                        });
                    });
                }, 5000, $scope);

                Timer.add("PreAuthInterval", false, function($scope) {
                    angular.forEach($scope.$storage.PreAuth, function(bike, key) {
                        bike.PreAuth.TimeoutLeft = bike.PreAuth.Timeout - Math.floor((new Date().getTime() - bike.PreAuth.CreatedAt.getTime()) / 1000);
                        if (bike.PreAuth.TimeoutLeft < 1) {
                            $scope.DeletePreAuth(bike.PreAuth.Id);
                            if ($scope.$storage.PreAuth.length == 0) {
                                $scope.clearPreAuthInterval();
                            }

                            $translate('pages.GetMapDataEx.PreAuth.Expired', function(string) {
                                Toast.show(string, 'long');
                            });
                        }
                    });
                    if ($scope.$storage.PreAuth.length == 0) {
                        Timer.remove("PreAuthInterval");
                    }
                }, 1000, $scope);

                angular.forEach($scope.$storage.PreAuth, function(bike, key) {
                    if (bike.PreAuth.TimeoutLeft < 1) {
                        $scope.DeletePreAuth(bike.PreAuth.Id);
                    }
                });
            }

            $scope.DeletePreAuth = function(reservationId) {
                angular.forEach($scope.$storage.PreAuth, function(bike, index) {
                    if (bike.PreAuth.Id == reservationId) {
                        $scope.$storage.PreAuth.splice(index, 1);
                    }
                });

                if ($scope.$storage.PreAuth.length == 0) {
                    Timer.remove("PreAuthInterval");
                }

                $scope.RefreshStationDetail();

                setTimeout(function() {
                    GetMapDataExService.get();
                }, 3000);
            }

            $scope.PreAuthCancel = function(bike) {
                ServerConnection.PreAuthCancel({
                    reservationId: bike.PreAuth.Id
                }, function(data) {
                    $scope.DeletePreAuth(bike.PreAuth.Id);
                    if ($scope.$storage.PreAuth.length == 0) {
                        $scope.clearPreAuthInterval();
                    }

                    $translate('pages.GetMapDataEx.PreAuth.4').then(function(string) {
                        Toast.show(string, "long");
                    });
                });
            }

            $scope.clearPreAuthInterval = function() {
                Timer.remove("PreAuthInterval");
                Timer.remove("PreAuthChecker");
            }

            $scope.GetBikesAtLocation = function() {
                if (typeof $scope.stationData == "object") {
                    GetBikesAtLocation.query({
                        locationId: $scope.stationData.LocationID
                    }, function(data) {
                        $scope.stationData.AvailableExternalBikes = data.filter(function(it) {
                            return it.AvailabilityCode == 1;
                        });
                    });

                    if ($rootScope.UserLocationError.code != 1 || ($rootScope.UserLocationError.error_code && $rootScope.UserLocationError.error_code != 'service_denied')) {
                        GetLocation.locate(function() {
                            $scope.computeDistance();
                        });
                    }
                }
            }

            $scope.showReturnList = function() {
                $ionicScrollDelegate.resize();
                $scope.$storage.temp.RentReturnList = "return";
            }

            $scope.showRentList = function() {
                $ionicScrollDelegate.resize();
                $scope.$storage.temp.RentReturnList = "rent";
            }


            $scope.PreAuthCreate = function(bike) {
                $scope.bikeLoading = bike;
                bike.loading = true;
                ServerConnection.PreAuthCreate({
                    bikeId: bike.BikeID
                }, function(data) {
                    data.CreatedAt = new Date();
                    bike.PreAuth = data;
                    bike.PreAuth.LocationID = $scope.stationData.LocationID;
                    angular.forEach($scope.stationData.Status.bikes, function(item, key) {
                        if (item.BikeID == bike.BikeID) {
                            $scope.stationData.Status.bikes.splice(key, 1);
                        }
                    });
                    $scope.$storage.PreAuth.push(bike);
                    bike.loading = false;

                    $scope.verifyPreAuth();

                    if (data.StateCode != 1) {
                        $translate('pages.GetMapDataEx.PreAuth.' + data.StateCode).then(function(string) {
                            Toast.show(string, "long");
                        });
                    }
                });
            }

            $scope.logout = function() {
                $scope.$parent.closeModal();
                $state.go("login");
            }

            /**
             * Zjistit, zda je mozne delat operace na stanici
             * @return {[type]} [description]
             */
            $scope.computeDistance = function() {
                if (typeof $scope.stationData != "object") return;
                var lat = false;
                var lng = false;
                var max = 0;
                if (!$scope.stationData.hasOwnProperty('LocationID')) {
                    lat = $scope.stationData.position.lat;
                    lng = $scope.stationData.position.lng;
                    max = AppConfig.BackendSettings.RENTAL_EXTERNAL_MAX_BIKE_DISTANCE_METRES;
                } else {
                    lat = $scope.stationData.Latitude;
                    lng = $scope.stationData.Longitude;
                    max = AppConfig.BackendSettings.RENTAL_EXTERNAL_MAX_LOCATION_DISTANCE;
                }

                var position = $rootScope.UserLocation;

                if (position !== false) {
                    var distance = parseInt(getDistanceFromLatLonInKm(position.latLng.lat, position.latLng.lng, lat, lng) * 1000);
                    $scope.stationData.distance = distance;

                    if (distance <= parseInt(AppConfig.BackendSettings.RESERVATION_PREAUTH_DISTANCE)) {
                        $scope.AllowDistancePreAuth = true;
                        $scope.messageTooFarFromStationPreAuth = false;
                    } else {
                        $scope.AllowDistancePreAuth = false;
                        $translate(['general.m', 'pages.GetMapDataEx.RentReturn.tooFarFromStationPreAuth']).then(function(string) {
                            var message = string['pages.GetMapDataEx.RentReturn.tooFarFromStationPreAuth'].replace("#MaximumDistance#", AppConfig.BackendSettings.RESERVATION_PREAUTH_DISTANCE + " " + string['general.m']);
                            message = message.replace("#UserDistance#", distance + " " + string['general.m']);
                            $scope.messageTooFarFromStationPreAuth = message;
                        });
                    }

                    if (distance <= parseInt(max)) {
                        $scope.messageTooFarFromStation = '';
                        $scope.AllowDistance = true;
                    } else {
                        $scope.AllowDistance = false;
                        $translate(['general.m', 'pages.GetMapDataEx.RentReturn.tooFarFromStation']).then(function(string) {
                            var message = string['pages.GetMapDataEx.RentReturn.tooFarFromStation'].replace("#MaximumDistance#", max + " " + string['general.m']);
                            message = message.replace("#UserDistance#", distance + " " + string['general.m']);
                            $scope.messageTooFarFromStation = message;
                        });
                    }
                }

                if ($rootScope.UserLocationError !== false && position === false) {
                    $translate('error.geolocation.loading').then(function(string) {
                        $scope.messageTooFarFromStation = string;
                    });
                    $scope.AllowDistance = false;
                }
            }
        }
    }
}])

.directive('bugReportSection', [function() {
    return {
        restrict: 'E', // E = Element, A = Attribute, C = Class, M = Comment
        templateUrl: function(el, attr) {
            return 'templates/bugReportSection-button.html';
        },
        scope: {
            path: '=',
            report: '=',
            finished: '='
        },
        replace: true,
        transclude: true,
        link: function(scope, iElm, iAttrs, controller) {
            scope.options = [];

            scope.level1 = iAttrs.level1;
            scope.level2 = iAttrs.level2;
            scope.source = JSON.parse(iAttrs.source);

            if (scope.source.hasOwnProperty("type")) {
                scope.type = scope.source.type;
                if (scope.type == "checklist") {
                    if (scope.source.options == "bikeProblem") {
                        scope.assembleBikeProblem();
                    }
                }
                if (scope.type == "list") {
                    if (scope.source.options == "lockList") {
                        scope.assembleLockList();
                    }
                    if (scope.source.options == "bikeList") {
                        scope.assembleBikeList();
                    }
                    if (scope.source.options == "locationList") {
                        scope.assembleLocationList();
                    }
                }
            } else {
                scope.type = "button";
            }
        },
        controller: function($scope, $localStorage, ServerConnection, $translate, GetMapDataEx, GetMapDataExService) {

            $scope.input = {};

            $scope.finished.spinner[$scope.level1] = false;

            $scope.$storage = $localStorage;

            $scope.addOption = function() {
                var option = $scope.input[$scope.level2];
                $scope.options.push({
                    id: option,
                    label: option
                });

                $scope.setPath({
                    uid: $scope.level1
                }, {
                    uid: $scope.level2
                }, {
                    uid: option
                });
            }

            $scope.checkbox = function(el, el2) {
                if (!$scope.report["checklist-detail"].hasOwnProperty(el)) {
                    $scope.report["checklist-detail"][el] = {};
                }
                if ($scope.report.checklist[el][el2]) {
                    $translate('pages.bugReport.tree.' + el2 + ".label").then(function(string) {
                        $scope.report["checklist-detail"][el][el2] = parseHtmlEnteties(string);
                    });
                } else {
                    delete $scope.report["checklist-detail"][el][el2];
                }
            }

            $scope.assembleBikeProblem = function() {
                $scope.report["checklist"] = {};
                $scope.options = $scope.source.child;
            }

            $scope.assembleLockList = function() {

                $translate(['pages.bugReport.tree.lock.default', 'pages.bugReport.tree.lock.option']).then(function(string) {
                    $scope.options.push({
                        id: "",
                        label: parseHtmlEnteties(string["pages.bugReport.tree.lock.default"])
                    });
                    var data = new Array(16);
                    for (var i = 1; i <= data.length; i++) {
                        $scope.options.push({
                            id: i.toString(),
                            label: parseHtmlEnteties(string['pages.bugReport.tree.lock.option']) + " " + i
                        });
                    }
                });
            }

            $scope.finishLocationList = function() {
                $scope.options = [];
                angular.forEach($scope.$storage.GetMapDataEx.mapData.Locations, function(Location) {
                    $scope.options.push({
                        id: Location.Name,
                        label: Location.Name
                    });
                });
                $scope.report["location"] = $scope.options[0].id;
                $scope.finished.spinner[$scope.level1] = false;
            }

            $scope.assembleLocationList = function() {

                // $scope.completed = 0;

                // $scope.finished.spinner[$scope.level1] = true;
                // $scope.finished.spinner[$scope.level1] = true;

                $scope.finishLocationList();
            }

            $scope.assembleBikeList = function() {
                if ($localStorage.anonymous == false) {
                    ServerConnection.GetCustomerRentalsServiceOngoing(function() {
                        $translate('pages.bugReport.tree.bike.default').then(function(string) {
                            $scope.options.push({
                                id: "",
                                label: string
                            });
                            angular.forEach($localStorage.GetCustomerRentals.ongoing.Rentals, function(TKRental) {
                                $scope.options.push({
                                    id: TKRental.BikeIdentifier,
                                    label: TKRental.BikeIdentifier
                                });
                            });
                        });
                    });
                }
            }

            $scope.setCheckList = function(key, value) {
                key = key.uid;
                value = value.uid;
                if (!$scope.report.hasOwnProperty(key)) {
                    $scope.report[key] = [];
                }

                $scope.report[key].push(value);
            }

            $scope.setPath = function(master, slave, value) {
                $scope.path[master.uid] = slave.uid;
                $scope.report[slave.uid] = value.uid;

                $scope.finished.last["slave"] = slave.uid;
                $scope.finished.last["master"] = master.uid;

                if ($scope.source.hasOwnProperty("child")) {
                    angular.forEach($scope.source['child'], function(item) {
                        if (item.type == "checklist") {
                            $scope.finished['done'] = true;
                        } else {
                            if (!$scope.report.hasOwnProperty(slave.uid)) {
                                $scope.finished['done'] = false;
                            }
                        }
                    });
                } else {
                    $scope.finished['done'] = true;
                }
            };

            $scope.showButtons = function(code) {
                var show = true;
                if (code.uid == 'position') return true;
                if ($scope.path.hasOwnProperty(code.uid) && $scope.path[code.uid] != '') {
                    return false;
                }

                return show;
            }

            $scope.branch = function(code) {
                if ($scope.report.hasOwnProperty(code.uid) && $scope.report[code.uid] != '') {
                    return true;
                }
                return false;
            }

        }
    };
}])

// .directive('returnBikeExternalButton', [function() {
//  return {
//    restrict: "E",
//    replace: true,
//    transclude: true,
//    template: "",
//    controller: function($scope, $state, $ionicHistory, Toast, $translate, $rootScope) {
//      $scope.returnBike = function(RentalID) {
//        if ($state.current.name == "menu.GetMapDataEx") {
//          $rootScope.$emit("switchRentReturn");
//        } else {
//          $ionicHistory.nextViewOptions({
//            disableBack: true
//          });
//          $translate("pages.GetMapDataEx.RentReturn.selectPlace4return").then(function(string) {
//            Toast.show(string);
//          });
//          $state.go("menu.GetMapDataEx",{
//            ReturnBikeExternal:RentalID
//          });
//        }
//      }
//    }
//  }
// }])

.directive('dashboardModule', function($compile) {
    return {
        restrict: "A",
        transclude: true,
        link: function(scope, elem, attrs) {
            scope.module = scope.$eval(attrs.module);
            scope.template = "<" + scope.module.element + "></" + scope.module.element + ">";
            var template = angular.element(scope.template);
            elem.append(template);
            $compile(template)(scope);
        }
    }
})

.directive('ongoingRentals', [function() {
    return {
        restrict: "E",
        replace: true,
        transclude: true,
        templateUrl: 'templates/listOngoingRentals.html',
        controller: function($scope, BatteryIcon, $rootScope, $state, Toast, $ionicHistory, $translate, $filter, $localStorage, ServerConnection, Timer) {
            $scope.$storage = $localStorage;
            $scope.offset = new Date().getTimezoneOffset();
            $scope.BatteryIcon = BatteryIcon;

            $scope.duration = function() {
                Timer.add("ongoingRentals", false, function($scope) {
                    angular.forEach($localStorage.GetCustomerRentals.ongoing.Rentals, function(el) {
                        var d = new Date(el.StartTime);
                        var duration = parseInt((new Date() - d.setMinutes(d.getMinutes() + $scope.offset)) / 1000);
                        el.Duration = $filter('date')($filter("secondsToDateTime")(duration), "HH:mm:ss");
                    });
                }, 1000, $scope);
            }

            $scope.$watch('$storage.GetCustomerRentals.ongoing.Rentals', function() {
                $scope.duration();
            });

            $scope.returnBike = function(RentalID) {
                $ionicHistory.nextViewOptions({
                    disableBack: true
                });
                $translate("pages.GetMapDataEx.RentReturn.selectPlace4return").then(function(string) {
                    Toast.show(string);
                });
                $scope.$storage.temp.RentReturnList = "return";

                $rootScope.GetMapDataEx.ReturnBikeExternal = RentalID;
                $state.go("menu.GetMapDataEx");
            }

            $scope.$on('doRefresh', function(event, args) {
                $scope.load();
            });

            $scope.load = function() {
                ServerConnection.GetCustomerRentalsServiceOngoing();
            }

            $scope.load();
        }
    }
}])

.directive('nearestStation', [function() {
    return {
        restrict: "E",
        replace: true,
        templateUrl: "templates/NearestStation.html",
        controller: function($scope, $rootScope, $localStorage, GetMapDataExService, $filter) {
            $scope.stationData = false;
            $scope.LocationID = false;
            $scope.showStationInfo = false;

            $rootScope.$on('PositionChanged', function(event, args) {
                if ($localStorage.GetMapDataEx.mapData.hasOwnProperty("Locations") && $localStorage.GetMapDataEx.mapData.Locations.length > 0) {
                    $scope.LocationID = $localStorage.GetMapDataEx.mapData.Locations[0].LocationID;
                    $scope.$broadcast("RefreshStationDetail");
                }
            });
        }
    }
}])

.directive('ebikeMotorProfiles', [function() {
    return {
        restrict: "E",
        replace: true,
        transclude: true,
        templateUrl: 'templates/listEbikeMotorProfiles.html',
        controller: function($scope, ServerConnection, $localStorage, EBikeMotorProfiles, Error) {
            $scope.$on('doRefresh', function(event, args) {
                $scope.load();
            });

            $scope.load = function() {
                if (!$localStorage.spinner.hasOwnProperty("EBikeMotorProfiles")) {
                    $localStorage.spinner.EBikeMotorProfiles = true;
                }

                ServerConnection.GetCustomerContractsServiceLoad();
                ServerConnection.GetCustomerEBikeSettings();

                $scope.$storage = $localStorage;
                $scope.UpdateMotorProfile = function() {
                    ServerConnection.GetCustomerServiceSave();
                }
            }
            $scope.load();
        }
    };
}])

.directive('getCustomerCards', [function() {
    return {
        restrict: "E",
        replace: true,
        transclude: true,
        templateUrl: 'templates/listGetCustomerCards.html',
        controller: function($scope, $localStorage, ServerConnection, Error) {
            $scope.$on('doRefresh', function(event, args) {
                $scope.load();
            });

            $scope.load = function() {
                if (!$localStorage.spinner.hasOwnProperty("GetCustomerCards")) {
                    $localStorage.spinner.GetCustomerCards = true;
                }

                ServerConnection.GetCustomerCardsQuery(function(data) {
                    $localStorage.spinner.GetCustomerCards = false;
                    $localStorage.TKCustomerCards = data;
                }, function(err) {
                    Error.solve(err);
                });
            }

            $scope.load();
        }
    };
}])

.directive('getCustomerContracts', [function() {
    return {
        restrict: "E",
        replace: true,
        transclude: true,
        templateUrl: 'templates/listGetCustomerContracts.html',
        controller: function($scope, ServerConnection) {
            $scope.$on('doRefresh', function(event, args) {
                $scope.load();
            });
            $scope.load = function() {
                ServerConnection.GetCustomerContractsServiceLoad();
            }
            $scope.load();
        }
    };
}])

.directive('listGetMapDataEx', [function() {
    return {
        restrict: "E",
        replace: true,
        transclude: true,
        templateUrl: 'templates/listGetMapDataEx.html',
        controller: function(GetMapDataExService, $scope) {
            $scope.$on('doRefresh', function(event, args) {
                $scope.load();
            });

            $scope.load = function() {
                GetMapDataExService.get();
            }

            $scope.load();
        }
    };
}])

.directive('languageSwitcher', [function() {
    return {
        restrict: "E",
        replace: true,
        transclude: true,
        templateUrl: 'templates/languageSwitcher.html',
        controller: function($scope, $ionicHistory, $state, $rootScope, $localStorage, $translate, tmhDynamicLocale) {
            $scope.$storage = $localStorage;

            $scope.changeLanguage = function(index) {
                $scope.$storage.lang = index;
                $translate.use(index);
                tmhDynamicLocale.set(index);

                var view = $ionicHistory.currentView();
                $state.go(view.stateName);

                if (index == "ar") {
                    $rootScope.dir = "rtl";
                    $rootScope.iconPosition = 'item-icon-right';
                } else {
                    $rootScope.dir = "ltr";
                    $rootScope.iconPosition = 'item-icon-left';
                }
            }
        }
    };
}])

.directive('apiIframe', function() {
    var linkFn = function(scope, element, attrs) {
        element.find('iframe').bind('load', function(event, x) {
            scope.$parent.iframeLoaded();
            event.target.contentWindow.scrollTo(0, 400);
        });
    };

    return {
        restrict: 'EA',
        scope: {
            src: '&src',
            height: '@height',
            width: '@width',
            scrolling: '@scrolling'
        },
        template: '<iframe id="apiIframe" class="frame" height="{{height}}" width="{{width}}" frameborder="0" border="0" marginwidth="0" marginheight="0" scrolling="{{scrolling}}" src="{{src()}}"></iframe>',
        link: linkFn
    };
})

.directive('distance', function() {
    return {
        restrict: "E",
        replace: true,
        scope: {
            distance: '@value',
        },
        template: "<span>{{distance}} {{'general.m'|translate}}</span>"
    }
})

.directive('ddCollapseText', ['$compile', '$ionicScrollDelegate', function($compile, $ionicScrollDelegate) {
    return {
        restrict: 'A',
        replace: true,
        link: function(scope, element, attrs) {

            // start collapsed
            scope.collapsed = false;

            // create the function to toggle the collapse
            scope.toggle = function() {
                scope.collapsed = !scope.collapsed;
                $ionicScrollDelegate.resize();
            };

            // get the value of the dd-collapse-text attribute
            attrs.$observe('ddCollapseText', function(maxLength) {
                // get the contents of the element
                var text = element.html();

                if (text.length > maxLength) {
                    // split the text in two parts, the first always showing
                    var firstPart = String(text).substring(0, maxLength);
                    var secondPart = String(text).substring(maxLength, text.length);

                    // create some new html elements to hold the separate info
                    var firstSpan = $compile('<span>' + firstPart + '</span>')(scope);
                    var secondSpan = $compile('<span ng-if="collapsed">' + secondPart + '</span>')(scope);
                    var moreIndicatorSpan = $compile('<span ng-if="!collapsed">&nbsp;...</span>')(scope);
                    var toggleButton = $compile('<div ng-click="toggle()" class="collapse-text-toggle fr"><span ng-if="collapsed">' + '<i class="fa fa-chevron-circle-up" aria-hidden="true"></i>' + '</span><span ng-if="!collapsed">' + '<i class="fa fa-chevron-circle-down" aria-hidden="true"></i>' + '</span></div>')(scope);

                    // remove the current contents of the element
                    // and add the new ones we created
                    element.empty();
                    element.append(firstSpan);
                    element.append(secondSpan);
                    element.append(moreIndicatorSpan);
                    element.append(toggleButton);

                }
            });
        }
    };
}]);