angular.module('app.filters', [])
    .filter('centerizeAmount', function() {
        return function(amount) {
            if (amount == null) amount = 0;
            if (amount > 0) {
                return amount;
            } else {
                return amount + "\u00A0";
            }
        };
    })

.filter('reverse', function() {
    return function(items) {
        return items.slice().reverse();
    };
})

.filter('cut', function() {
    return function(value, wordwise, max, tail) {
        if (!value) return '';

        max = parseInt(max, 10);
        if (!max) return value;
        if (value.length <= max) return value;

        value = value.substr(0, max);
        if (wordwise) {
            var lastspace = value.lastIndexOf(' ');
            if (lastspace != -1) {
                //Also remove . and , so its gives a cleaner result.
                if (value.charAt(lastspace - 1) == '.' || value.charAt(lastspace - 1) == ',') {
                    lastspace = lastspace - 1;
                }
                value = value.substr(0, lastspace);
            }
        }

        return value + (tail || '');
    };
})

.filter('balance', function(AppConfig, $filter) {
    return function(price) {
        if (AppConfig.currency.placement === false) {
            return $filter('currency')(price, '') + " " + AppConfig.currency.symbol;
        } else {
            return AppConfig.currency.symbol + $filter('currency')(price, '');
        }
    };
})

.filter('isEmpty', function() {
    var bar;
    return function(obj) {
        for (bar in obj) {
            if (obj.hasOwnProperty(bar)) {
                return false;
            }
        }
        return true;
    };
})

.filter('secondsToDateTime', [function() {
    return function(seconds) {
        return new Date(1970, 0, 1).setSeconds(seconds);
    };
}])

.filter('orderObjectBy', function() {
    return function(input, attribute) {
        if (!angular.isObject(input)) return input;

        var array = [];
        for (var objectKey in input) {
            array.push(input[objectKey]);
        }

        array.sort(function(a, b) {
            a = parseInt(a[attribute]);
            b = parseInt(b[attribute]);
            return a - b;
        });
        return array;
    }
});;
