var HS = angular.module('app.services', []);

function parseHtmlEnteties(str) {
    if (str.length > 0) {
        return str.replace(/&#([0-9]{1,3});/gi, function (match, numStr) {
            var num = parseInt(numStr, 10); // read num as normal number
            return String.fromCharCode(num);
        });
    } else {
        return str;
    }
}

function getDistanceFromLatLonInKm(lat1, lon1, lat2, lon2) {
    var R = 6371; // Radius of the earth in km
    var dLat = deg2rad(lat2 - lat1); // deg2rad below
    var dLon = deg2rad(lon2 - lon1);
    var a =
        Math.sin(dLat / 2) * Math.sin(dLat / 2) +
        Math.cos(deg2rad(lat1)) * Math.cos(deg2rad(lat2)) *
        Math.sin(dLon / 2) * Math.sin(dLon / 2);
    var c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
    var d = R * c; // Distance in km
    return d;
}

HS.service("BatteryIcon", function (AppConfig) {

    this.info = function (battery, type) {
        type = type || "icon";
        var icon = "ion-battery-charging balanced";
        var state = "max";
        if (battery < AppConfig.batteryStates.max) {
            state = "high";
            icon = "ion-battery-full balanced";
        }
        if (battery < AppConfig.batteryStates.high) {
            state = "medium"
            icon = "ion-battery-half energized";
        }
        if (battery < AppConfig.batteryStates.medium) {
            state = "low"
            icon = "ion-battery-low energized";
        }
        if (battery < AppConfig.batteryStates.low) {
            state = "critical";
            icon = "ion-battery-low assertive";
        }
        if (battery < AppConfig.batteryStates.critical) {
            state = "empty";
            icon = "ion-battery-empty assertive";
        }

        if (type == "state") {
            return state;
        }

        if (type == "icon") {
            return icon;
        }
    }
});

function deg2rad(deg) {
    return deg * (Math.PI / 180)
}

HS.getParameterByName = function (name, url) {
    if (!url) url = window.location.href;
    name = name.replace(/[\[\]]/g, "\\$&");
    var regex = new RegExp("[?&]" + name + "(=([^&#]*)|&|#|$)"),
        results = regex.exec(url);
    if (!results) return null;
    if (!results[2]) return '';
    return decodeURIComponent(results[2].replace(/\+/g, " "));
}

HS.serialize = function (obj) {
    var str = [];
    for (var p in obj)
        if (obj.hasOwnProperty(p)) {
            str.push(encodeURIComponent(p) + "=" + encodeURIComponent(obj[p]));
        }
    return str.join("&");
}

HS.service('GetInfosService', function (GetInfos, $localStorage) {
    this.load = function () {
        GetInfos.get({
            lang: $localStorage.lang
        }, function (data) {
            $localStorage.Infos = data;
        });
    }
});

HS.factory('GetInfos', function ($resource, AppConfig, $rootScope) {
    return $resource(AppConfig.backend.url + "Infos", {
        lang: '@lang'
    }, {
        get: {
            params: {
                externalClientID: AppConfig.backend[$rootScope.platform].client_id,
                pageNumber: 1
            }
        }
    });
});

HS.service('GetNewsService', function (GetNews, $localStorage) {
    this.load = function () {
        GetNews.get({
            lang: $localStorage.lang
        }, function (data) {
            $localStorage.News = data;
        });
    }
});

HS.factory('GetNews', function ($resource, AppConfig, $rootScope) {
    return $resource(AppConfig.backend.url + "News", {
        lang: '@lang'
    }, {
        get: {
            params: {
                externalClientID: AppConfig.backend[$rootScope.platform].client_id,
                pageNumber: 1
            }
        }
    });
});

/**
 * casovac pro ruzne setInterval pozadavky, bezi pouze jeden casovat a obsluhuje vsechny registrovane pozadavky
 */
HS.service('Timer', function ($interval, $localStorage) {
    this.interval = false;
    this.requests = [];
    this.counter = 0;
    this.step = 1000;

    this.init = function () {
        var self = this;
        if (self.interval !== false) self.stop();
        this.interval = $interval(function () {
            // var background = cordova.plugins.backgroundMode.isActive();
            var background = false;
            self.counter += self.step;
            angular.forEach(self.requests, function (request) {
                if (background && request.background !== true) return;
                if ((self.counter % request.interval == 0) || (background && request.background === true)) {
                    request.callback(request.scope);
                }
            });
        }, self.step);
    }

    this.add = function (name, background, callback, interval, scope) {
        var add = true;
        angular.forEach(this.requests, function (el, key) {
            if (el.name == name) {
                add = false;
                return;
            }
        });

        if (add === false) return;
        this.requests.push({
            "name": name,
            "interval": interval,
            "callback": callback,
            "scope": scope,
            "background": background
        });

        if (this.interval === false) {
            this.init();
        }
    }

    /**
     * Nastavit interval na dvounasobek requestu s background modem
     * @return {[type]} [description]
     */
    this.backgroundMode = function () {
        var self = this;
        angular.forEach(this.requests, function (el, key) {
            if (el.background === true) {
                if (el.interval < self.step || self.step === 1000) self.step = (el.interval * 2);
            }
        });
        self.init();
    }

    /**
     * background mode byl deaktivovan, prejit do klasickeho rezimu
     * @return {[type]} [description]
     */
    this.foregroundMode = function () {
        var self = this;
        self.step = 1000;
        self.init();
    }

    this.resetAll = function () {
        this.requests = [];
        self.stop();
    }

    this.remove = function (name) {
        var self = this;

        angular.forEach(this.requests, function (el, key) {
            if (el.name == name) {
                self.requests.splice(key, 1);
            }
        });

        if (this.requests.length === 0) {
            this.stop();
        }
    }

    this.stop = function () {
        if (this.interval !== false) {
            $interval.cancel(this.interval);
            this.interval = false;
            this.counter = 0;
        } else {
        }
    }
});

HS.service('ServerConnection', function ($http, $state, PreAuth, $ionicLoading, $localStorage, AppConfig, $rootScope, GetCustomerService, GetCustomerRentalsService, RentReturnBikeExternal, GetCustomerCards, GetCustomerAccountActivitiesService, GetCustomerContractsService, EBikeMotorProfiles) {
    this.token = function (type, callback) {
        var date = new Date();
        var UTCDate = new Date(date.getUTCFullYear(), date.getUTCMonth(), date.getUTCDate(), date.getUTCHours(), date.getUTCMinutes(), date.getUTCSeconds());
        var expiry = new Date($localStorage.OAuth['.expires']);
        var UTCExpiry = new Date(expiry.getUTCFullYear(), expiry.getUTCMonth(), expiry.getUTCDate(), expiry.getUTCHours(), expiry.getUTCMinutes(), expiry.getUTCSeconds());

        if (UTCExpiry.getTime() > UTCDate.getTime()) {
            callback();
            return;
        } else {
            var params = {};
            if (angular.isUndefined($localStorage.Login)) return;
            params.username = $localStorage.Login.CardNumber;
            params.password = $localStorage.Login.pin;
            params.grant_type = "password";

            params.client_id = AppConfig.backend[$rootScope.platform].client_id;
            params.client_secret = AppConfig.backend[$rootScope.platform].client_secret;

            var request = {
                method: "POST",
                url: AppConfig.backend.url + "Token",
                headers: {
                    'Content-Type': 'application/x-www-form-urlencoded'
                },
                data: HS.serialize(params),
                timeout: 10000
            };

            delete $http.defaults.headers.common['Authorization'];
            $http(request).then(function (data) {
                $localStorage.OAuth = data.data;
                $localStorage.anonymous = false;

                $http.defaults.headers.common['Authorization'] = 'Bearer ' + $localStorage.OAuth.access_token;
                $ionicLoading.hide();
                callback();
            }, function (err) {
                $localStorage.OAuth = false;
                $state.go('login');
                $ionicLoading.hide();
                return false;
            });
        }
    }

    this.GetCustomerEBikeSettings = function (callback) {
        this.token("EBikeMotorProfiles", function () {
            EBikeMotorProfiles.query(function (data) {
                $localStorage.spinner.EBikeMotorProfiles = false;
                $localStorage.EBikeMotorProfiles = data;
            });
        });
    }

    this.GetCustomerRentalsServiceOngoing = function (callback) {
        if ($localStorage.anonymous == false) {
            this.token("GetCustomerRentalsServiceOngoing", function () {
                GetCustomerRentalsService.ongoing(callback);
            });
        }
    }
    this.GetCustomerRentalsServiceClosed = function (callback, pageNumber) {
        pageNumber = pageNumber || 1;
        this.token("GetCustomerRentalsServiceClosed", function () {
            GetCustomerRentalsService.closed(callback, pageNumber);
        });
    }
    this.RentReturnBikeExternalRent = function (params, callback) {
        this.token("RentReturnBikeExternalRent", function () {
            RentReturnBikeExternal.rent(params, callback);
        });
    }
    this.RentReturnBikeExternalReturn = function (params, callback) {
        this.token("RentReturnBikeExternalReturn", function () {
            RentReturnBikeExternal.return(params, callback);
        });
    }
    this.PreAuthCreate = function (params, callback) {
        this.token("PreAuthCreate", function () {
            PreAuth.create(params, callback);
        });
    }
    this.PreAuthCancel = function (params, callback) {
        this.token("PreAuthCancel", function () {
            PreAuth.cancel(params, callback);
        });
    }
    this.PreAuthDetail = function (params, callback) {
        this.token("PreAuthDetail", function () {
            PreAuth.detail(params, callback);
        });
    }
    this.GetCustomerCardsQuery = function (callback) {
        this.token("GetCustomerCardsQuery", function () {
            GetCustomerCards.query(callback);
        });
    }
    this.GetCustomerAccountActivitiesServiceLoad = function (callback, pageNumber) {
        pageNumber = pageNumber || false;
        this.token("GetCustomerAccountActivitiesServiceLoad", function () {
            GetCustomerAccountActivitiesService.load(callback, pageNumber);
        });
    }
    this.GetCustomerServiceGet = function (callback) {
        this.token("GetCustomerServiceGet", function () {
            GetCustomerService.get(callback);
        });
    }
    this.GetCustomerServiceSave = function () {
        this.token("GetCustomerServiceSave", function () {
            GetCustomerService.save();
        });
    }
    this.GetCustomerContractsServiceLoad = function () {
        this.token("GetCustomerContractsService", function () {
            GetCustomerContractsService.load();
        });
    }
});

HS.service('cordovaLaunchNavigator', ['$q', function ($q) {
    "use strict";

    var cordovaLaunchNavigator = {};
    cordovaLaunchNavigator.navigate = function (destination, options) {
        var q = $q.defer(),
            isRealDevice = ionic.Platform.isWebView();

        if (!isRealDevice) {
            q.reject("launchnavigator will only work on a real mobile device! It is a NATIVE app launcher.");
        } else {
            try {

                var successFn = options.successCallBack || function () {
                    },
                    errorFn = options.errorCallback || function () {
                    },
                    _successFn = function () {
                        successFn();
                        q.resolve();
                    },
                    _errorFn = function (err) {
                        errorFn(err);
                        q.reject(err);
                    };

                options.successCallBack = _successFn;
                options.errorCallback = _errorFn;

                launchnavigator.navigate(destination, options);
            } catch (e) {
                q.reject("Exception: " + e.message);
            }
        }
        return q.promise;
    };
    return cordovaLaunchNavigator;
}]);

HS.service('Speedometer', function ($rootScope, $localStorage) {
    this.update = function (position) {
        if (!position.hasOwnProperty("coords")) {
            $rootScope.UserSpeedometer.speed = 0;
            $rootScope.UserSpeedometer.altitude = "?";
            return;
        }

        if (!position.coords.hasOwnProperty("speed")) {
            $rootScope.UserSpeedometer.speed = 0;
        } else {
            $rootScope.UserSpeedometer.speed = position.coords.speed;
        }

        if (parseFloat($rootScope.UserSpeedometer.speed) < 0.8) {
            $rootScope.UserSpeedometer.speed = 0;
        }

        if (!position.coords.hasOwnProperty("altitude") || isNaN(position.coords.altitude)) {
            $rootScope.UserSpeedometer.altitude = "?";
        } else {
            $rootScope.UserSpeedometer.altitude = position.coords.altitude;
        }

        if ($localStorage.distanceUnits == "mi") {
            $rootScope.UserSpeedometer.speed = $rootScope.UserSpeedometer.speed * 2.236936;
        }

        if ($localStorage.distanceUnits == "km") {
            $rootScope.UserSpeedometer.speed = $rootScope.UserSpeedometer.speed * 3.6;
        }

        $rootScope.UserSpeedometer.speed = Math.round($rootScope.UserSpeedometer.speed);
        $rootScope.UserSpeedometer.altitude = Math.round($rootScope.UserSpeedometer.altitude);
    }
});

HS.service('GetLocation', function ($rootScope, Toast, Speedometer, $cordovaGeolocation, $localStorage) {
    this.locate = function (callback) {
        callback = callback || false;
        var options = {
            enableHighAccuracy: true
        };

        if ($rootScope.map === false) {
            $cordovaGeolocation.getCurrentPosition(options).then(
                function (position) {
                    Speedometer.update(position);

                    var position = {
                        "latLng": {
                            "lat": position.coords.latitude,
                            "lng": position.coords.longitude
                        }
                    };
                    $rootScope.UserLocation = position;
                    if (callback !== false) callback(position);
                },
                function (err) {
                    $rootScope.UserLocationError = err;
                    if (callback !== false) callback(false);
                });
        } else {
            $rootScope.map.getMyLocation(function (position) {
                position.coords = {
                    speed: position.speed,
                    altitude: position.altitude
                };
                Speedometer.update(position);
                $rootScope.UserLocation = position;
                if (callback !== false) callback(position);
            }, function (err) {
                $rootScope.UserLocationError = err;
            });
        }
    }
});

HS.service('GetFaqsService', function (GetFaqs, $localStorage) {
    this.load = function () {
        GetFaqs.get({
            lang: $localStorage.lang
        }, function (data) {
            $localStorage.Faqs = data;
        });
    }
});

HS.factory('GetFaqs', function ($resource, AppConfig, $rootScope) {
    return $resource(AppConfig.backend.url + "Faqs", {
        lang: '@lang'
    }, {
        get: {
            params: {
                externalClientID: AppConfig.backend[$rootScope.platform].client_id,
                pageNumber: 1
            }
        }
    });
});

HS.service('GetContactsService', function (GetContacts, $localStorage) {
    this.load = function () {
        GetContacts.get({
            lang: $localStorage.lang
        }, function (data) {
            $localStorage.Contacts = data;
        });
    }
});

HS.factory('GetContacts', function ($resource, AppConfig, $rootScope) {
    return $resource(AppConfig.backend.url + "Contacts", {
        lang: '@lang'
    }, {
        get: {
            params: {
                externalClientID: AppConfig.backend[$rootScope.platform].client_id,
                pageNumber: 1
            }
        }
    });
});

HS.service('FrontendService', function ($translate, $ionicLoading, $localStorage, AppConfig, $http, $ionicLoading, Toast, $rootScope) {

    this.api = AppConfig.url + "api/";

    this.requireToken = ["add-contract", "bug-report"];

    this.get = function (endpoint, params, callback, error, fault) {
        endpoint = endpoint || false;
        if (endpoint === false) return;

        params = params || {};
        params["Lang"] = $localStorage.lang;
        params["externalClientId"] = AppConfig.backend[$rootScope.platform].client_id;
        callback = callback || false;
        error = error || false;
        fault = fault || false;

        var request = {
            method: "GET",
            url: this.api + endpoint + "?" + HS.serialize(params),
            headers: {
                'Content-Type': 'application/x-www-form-urlencoded'
            },
            timeout: 15000
        };

        var _this = this;

        $http(request).then(function (data) {
            callback(data.data.data);
        }, function (err) {
            _this.error(err);
            if (error === false) {
                // console.log(err);
                $ionicLoading.hide();
            } else {
                error(err);
            }
        }).catch(function (err) {
            _this.error(err);
            if (fault === false) {
                console.log(err);
                $ionicLoading.hide();
            } else {
                fault(err);
            }
        });
    }

    this.post = function (endpoint, params, callback, error, fault) {
        endpoint = endpoint || false;
        if (endpoint === false) return;

        params = params || {};
        callback = callback || false;
        error = error || false;
        fault = fault || false;

        var headers = {
            'Content-Type': 'application/json'
        };

        if (this.requireToken.indexOf(endpoint) !== -1) {
            headers.authorization = "Bearer " + $localStorage.FEToken.token
        }

        var _this = this;

        params["Lang"] = $localStorage.lang;

        var request = {
            method: "POST",
            url: this.api + endpoint + "?" + HS.serialize(params),
            headers: headers,
            data: params,
            timeout: 45000
        };
        $http(request).then(function (data) {
            callback(data.data.data);
        }, function (err) {
            _this.error(err);
            if (error === false) {
                console.log(err);
            } else {
                error(err);
            }
        }).catch(function (err) {
            _this.error(err);
            if (fault === false) {
                console.log(err);
            } else {
                fault(err);
            }
        });
    }

    this.error = function (err) {
        if (err && err.hasOwnProperty("data") && err.data.hasOwnProperty("errors")) {
            Toast.show(err.data.errors[0].detail);
        }
    }

    this.auth = function () {
        var params = {
            CardNumber: $localStorage.Login.CardNumber,
            Pin: $localStorage.Login.pin
        };
        var request = {
            method: "POST",
            url: this.api + "auth",
            headers: {
                'Content-Type': 'application/x-www-form-urlencoded'
            },
            data: HS.serialize(params),
            timeout: 15000
        };

        $http(request).then(function (data) {
            $localStorage.FEToken = data.data.data;
        }, function (err) {
            console.log(err);
        }).catch(function (err) {
            console.log(err);
        });
    }
});

HS.factory('ConnectivityMonitor', function ($rootScope, $cordovaNetwork, $translate, Toast) {
    return {
        isOnline: function () {
            if (ionic.Platform.isWebView()) {
                return $cordovaNetwork.isOnline();
            } else {
                return navigator.onLine;
            }
        },
        ifOffline: function () {
            if (ionic.Platform.isWebView()) {
                return !$cordovaNetwork.isOnline();
            } else {
                return !navigator.onLine;
            }
        },
        isOffline: function () {
            $translate("error.offline").then(function (string) {
                Toast.show(string);
            });
        }
    }
});

HS.service('Error', function ($ionicLoading, $translate, $state, $localStorage, Toast, AppConfig, $rootScope) {

    this.solve = function (err) {
        $ionicLoading.hide();
        $translate("error.status." + err.status).then(function (string) {
            Toast.show(string);
        });
    }
});

HS.factory('PreAuth', function ($resource, $localStorage, AppConfig, Toast, $rootScope, $translate) {
    return $resource(AppConfig.backend.url + "Reservation/Create", {
        bikeId: '@bikeId',
        spikeId: '@spikeId',
        reservationId: '@reservationId'
    }, {
        'create': {
            params: {
                lang: $localStorage.lang,
            },
            method: "POST",
            headers: {
                'Content-Type': 'application/json',
            },
            interceptor: {
                responseError: function (err) {
                    $rootScope.$broadcast('requestFinished');
                    Toast.show(err.data.Message);
                }
            }
        },
        'cancel': {
            params: {
                lang: $localStorage.lang,
            },
            method: "POST",
            url: AppConfig.backend.url + "Reservation/Cancel",
            headers: {
                'Content-Type': 'application/json',
            },
            interceptor: {
                response: function (response) {
                    return response;
                },

                responseError: function (err) {
                    $rootScope.$broadcast('requestFinished');
                    Toast.show(err.data.Message);
                }
            }
        },
        'detail': {
            params: {
                lang: $localStorage.lang,
            },
            method: "GET",
            url: AppConfig.backend.url + "Reservation",
            headers: {
                'Content-Type': 'application/json',
            },
            interceptor: {
                responseError: function (err) {
                    $rootScope.$broadcast('requestFinished');
                    Toast.show(err.data.Message);
                }
            }
        }
    });
});

HS.factory('RentReturnBikeExternal', function ($resource, $localStorage, AppConfig, Toast, $rootScope) {
    return $resource(AppConfig.backend.url + "Rental", {
        bikeId: '@bikeId',
        locationId: '@locationId',
        rentalId: '@rentalId',
        spikeId: '@spikeId',
        latitude: '@latitude',
        longitude: '@longitude',
        confirmationCode: '@confirmationCode'
    }, {
        'rent': {
            params: {
                lang: $localStorage.lang,
            },
            method: "POST",
            headers: {
                'Content-Type': 'application/json',
            },
            interceptor: {
                responseError: function (err) {
                    $rootScope.$broadcast('requestFinished');
                    Toast.show(err.data.Message);
                }
            }
        },
        'return': {
            params: {
                lang: $localStorage.lang,
            },
            url: AppConfig.backend.url + "Rental/ReturnBike",
            method: "PUT",
            headers: {
                'Content-Type': 'application/json',
            },
            interceptor: {
                responseError: function (err) {
                    $rootScope.$broadcast('requestFinished');
                    Toast.show(err.data.Message);
                }
            }
        }
    });
});

HS.factory('GetBikesAtLocation', function ($resource, $localStorage, AppConfig) {
    return $resource(AppConfig.backend.url + "Bike/GetBikesAtLocation", {
        locationId: "@locationId",
        origin: "hp"
    }, {
        query: {
            isArray: true,
            params: {
                lang: $localStorage.lang,
            }
        }
    });
});

HS.factory('GetCustomerRentals', function ($resource, $localStorage, AppConfig) {
    return $resource(AppConfig.backend.url + "User/Rentals", {}, {
        ongoing: {
            method: "GET",
            params: {
                lang: $localStorage.lang,
                statusCode: 1,
                pageNumber: 1
            },
        },
        closed: {
            method: "GET",
            params: {
                lang: $localStorage.lang,
                statusCode: 2,
                pageNumber: '@pageNumber'
            },
        }
    });
});

HS.factory('GetCustomerAccountActivities', function ($resource, AppConfig) {
    return $resource(AppConfig.backend.url + "User/AccountActivities", {
        lang: '@lang'
    }, {
        get: {
            params: {
                pageNumber: '@pageNumber'
            }
        }
    });
});

HS.factory('GetCustomerCards', function ($resource, $localStorage, AppConfig) {
    return $resource(AppConfig.backend.url + "User/Cards", {}, {
        query: {
            isArray: true,
            params: {
                lang: $localStorage.lang
            }
        }
    });
});

HS.factory('GetCustomerContracts', function ($resource, $localStorage, AppConfig) {
    return $resource(AppConfig.backend.url + "User/Contracts", {
        pageNumber: "@pageNumber"
    }, {
        query: {
            isArray: false,
            params: {
                lang: $localStorage.lang
            }
        }
    });
});

HS.factory('Setting', function ($resource, $localStorage, AppConfig) {
    return $resource(AppConfig.backend.url + "Setting", {}, {
        query: {
            transformResponse: function (data) {
                var response = {};
                data = angular.fromJson(data);
                angular.forEach(data, function (el) {
                    response[el.Key] = el.Value;
                });
                return response;
            }
        }
    });
});

HS.service('GetCustomerService', function (GetCustomer, Error, Toast, $localStorage, $translate) {
    this.get = function (callback) {
        GetCustomer.get(callback);
    }
    this.save = function (callback) {
        GetCustomer.save($localStorage.TKCustomer, function (data) {
            $translate("pages.EBikeMotorProfiles.saved").then(function (string) {
                Toast.show(string);
            });
        }, function (err) {
            Error.solve(err);
        });
    }
});


HS.factory('GetCustomer', function ($resource, $localStorage, $rootScope, AppConfig) {
    return $resource(AppConfig.backend.url + "User", {}, {
        get: {
            params: {
                lang: $localStorage.lang
            }
        },
        'save': {
            params: {
                lang: $localStorage.lang
            },
            method: "PUT",
            headers: {
                'Content-Type': 'application/json',
            }
        }
    });
});

HS.factory('GetMapDataEx', function ($resource, $localStorage, AppConfig) {
    return $resource(AppConfig.backend.url + "Location/Map/Extended", {}, {
        get: {
            params: {
                lang: $localStorage.lang
            }
        }
    });
});

HS.factory('GetBikesOutOfLocation', function ($resource, $localStorage, AppConfig) {
    return $resource(AppConfig.backend.url + "Bike/GetBikesOutOfLocation", {}, {
        query: {
            isArray: true,
            params: {
                lang: $localStorage.lang,
            }
        }
    });
});

HS.service('GetCustomerAccountActivitiesService', function (GetCustomerAccountActivities, Error, $localStorage) {
    this.load = function (callback, pageNumber) {
        callback = callback || false;
        pageNumber = pageNumber || 1;
        GetCustomerAccountActivities.get({
            "lang": $localStorage.lang,
            "pageNumber": pageNumber
        }, function (data) {
            if ($localStorage.TKAccountActivities == false) {
                $localStorage.TKAccountActivities = data;
            } else {
                $localStorage.TKAccountActivities.Activities = $localStorage.TKAccountActivities.Activities.concat(data.Activities);
            }

            if (callback !== false) {
                callback();
            }
        }, function (err) {
            Error.solve(err);
        });
    }
});

HS.service('GetCustomerContractsService', function (GetCustomerContracts, $filter, $localStorage, Error) {

    this.load = function (PageNumber) {
        if (!$localStorage.spinner.hasOwnProperty("GetCustomerContracts")) {
            $localStorage.spinner.GetCustomerContracts = true;
        }

        PageNumber = PageNumber || 1;
        PageTotal = 1;
        StatusCode = 2;
        scope = this;
        GetCustomerContracts.query({
                "pageNumber": PageNumber
            },
            function (data) {
                PageTotal = data.PageTotal;
                $localStorage.TKCustomerContracts = data;
                $localStorage.TKCustomerContracts.Contracts = $filter('filter')(data.Contracts, {
                    StatusCode: StatusCode
                });

                while (PageNumber < PageTotal) {
                    PageNumber++;
                    GetCustomerContracts.query({
                            "pageNumber": PageNumber
                        },
                        function (data) {
                            for (i in data.Contracts) {
                                var el = data.Contracts[i];
                                $localStorage.TKCustomerContracts.Contracts.push(el);
                            }
                        },
                        function (err) {
                            Error.solve(err);
                        });
                }
                $localStorage.spinner.GetCustomerContracts = false;
            },
            function (err) {
                Error.solve(err);
            });
    }
});

HS.service('GetCustomerRentalsService', function ($localStorage, $rootScope, $filter, $translate, BatteryIcon, GetCustomerRentals, Error, AppConfig) {
    var self = this;

    this.process = function (data, callback) {
        $localStorage.spinner.GetCustomerRentalsOngoing = false;
        $localStorage.GetCustomerRentals.ongoing = data;

        var background = false;

        angular.forEach($localStorage.GetCustomerRentals.ongoing.Rentals, function (item) {
            var TKBusinessRule = $filter('filter')($localStorage.GetBusinessRules, {
                Name: item.BusinessRuleName
            });

            if (!$localStorage.Notifications.hasOwnProperty(item.RentalID)) {
                $localStorage.Notifications[item.RentalID] = {
                    FreePeriod: false,
                    BatteryLevel: false
                };
            }

            if (TKBusinessRule.length == 1) {
                TKBusinessRule = TKBusinessRule[0];

                if (TKBusinessRule.hasOwnProperty("PriceIntervals") && TKBusinessRule.PriceIntervals.hasOwnProperty("TKPriceInterval") && angular.isArray(TKBusinessRule.PriceIntervals.TKPriceInterval)) {
                    var limit = TKBusinessRule.PriceIntervals.TKPriceInterval[0];


                    if (limit.Price < 1 && $localStorage.Notifications[item.RentalID].FreePeriod === false) {
                        var d = new Date(item.StartTime);
                        var duration = new Date() - (new Date(d.setMinutes(d.getMinutes() + new Date().getTimezoneOffset())));
                        if (duration < (parseInt(limit.Length) - 5) * 60 * 1000) return;
                        $localStorage.Notifications[item.RentalID].FreePeriod = true;
                        $translate(["pages.OngoingRentals.freeLimitTitle", "pages.OngoingRentals.freeLimit"]).then(function (string) {
                            // var media = new Media("img/bell.mp3");
                            // media.play();
                            // cordova.plugins.notification.local.schedule({
                            //     title: item.BikeIdentifier + ": " + string["pages.OngoingRentals.freeLimitTitle"],
                            //     text: string["pages.OngoingRentals.freeLimit"]
                            // });
                        });
                    }
                }
            }

            if (item.IsEbike) {
                background = true;

                var state = BatteryIcon.info(item.BatteryLevel, "state");

                if ($localStorage.Notifications[item.RentalID].BatteryLevel === false) {
                    $localStorage.Notifications[item.RentalID].BatteryLevel = 100;
                }

                if ($localStorage.Notifications[item.RentalID].BatteryLevel > AppConfig.batteryStates[state]) {
                    $localStorage.Notifications[item.RentalID].BatteryLevel = AppConfig.batteryStates[state];
                    $translate("pages.EBikeMotorProfiles.notifications." + state).then(function (string) {
                        // var media = new Media("img/bell.mp3");
                        // media.play();
                        string = string.replace("#range#", $rootScopwe.ebikeRange(item.BatteryLevel) + " " + $localStorage.distanceUnits);
                        // cordova.plugins.notification.local.schedule({
                        //     title: item.BikeIdentifier,
                        //     text: string
                        // });
                    });
                }
            }
        });

        // cordova.plugins.backgroundMode.setEnabled(background);

        if (callback !== false) callback();
    }

    this.ongoing = function (callback) {
        callback = callback || false;
        if (!$localStorage.spinner.hasOwnProperty("GetCustomerRentalsOngoing")) {
            $localStorage.spinner.GetCustomerRentalsOngoing = true;
        }

        var newDate = new Date();
        var check = newDate.setTime(newDate.getTime() - $localStorage.CacheSettings.GetCustomerRentalsOngoing);
        if ($localStorage.Cache.GetCustomerRentalsOngoing > check) {
            self.process($localStorage.GetCustomerRentals.ongoing, callback);
        } else {
            GetCustomerRentals.ongoing(function (data) {
                $localStorage.Cache.GetCustomerRentalsOngoing = new Date().getTime();
                self.process(data, callback);
            }, function (err) {
                Error.solve(err);
            });
        }
    }
    this.closed = function (callback, pageNumber) {
        pageNumber = pageNumber || 1;
        callback = callback || false;
        if (!$localStorage.spinner.hasOwnProperty("GetCustomerRentalsClosed")) {
            $localStorage.spinner.GetCustomerRentalsClosed = true;
        }

        GetCustomerRentals.closed({
            pageNumber: pageNumber
        }, function (data) {
            $localStorage.spinner.GetCustomerRentalsClosed = false;
            $localStorage.GetCustomerRentals.closed = data;
            if (callback !== false) callback();
        }, function (err) {
            Error.solve(err);
        });
    }
});

HS.service('GetMapDataExService', function (AppConfig, $localStorage, GetMapDataEx, GetBikesOutOfLocation, $rootScope, $filter, $translate, $timeout) {
    var self = this;

    // prepocitat vzdalenosti od uzivatele ke vsem stanicim
    this.distance = function (callback) {

        callback = callback || false;

        var position = $rootScope.UserLocation;

        if (position !== false) {
            angular.forEach($localStorage.GetMapDataEx.mapData.Locations, function (Location) {
                var distance = parseInt(getDistanceFromLatLonInKm(position.latLng.lat, position.latLng.lng, Location.Latitude, Location.Longitude) * 1000);
                Location.distance = distance;
            });
            $localStorage.GetMapDataEx.mapData.Locations = $filter('orderBy')($localStorage.GetMapDataEx.mapData.Locations, "distance");
            if (callback !== false) callback();
        } else {
            console.log('position not available');
        }
    }
    this.get = function (callback) {
        callback = callback || false;

        var toUpdate = 1;
        if (AppConfig.BackendSettings.RENTAL_ALLOW_EXTERNAL_RETURN_ANYWHERE == "1" || AppConfig.BackendSettings.RENTAL_ALLOW_EXTERNAL_RENTAL_ANYWHERE == "1") {
            toUpdate++;
        }

        var ShowBikeAvailabilityTypes = ['AvailableBikes'];
        if ($localStorage.FrontendSettings && $localStorage.FrontendSettings.Map && $localStorage.FrontendSettings.Map.ShowBikeAvailabilityTypes) {
            ShowBikeAvailabilityTypes = ShowBikeAvailabilityTypes.concat($localStorage.FrontendSettings.Map.ShowBikeAvailabilityTypes);
        }

        GetMapDataEx.get(function (data) {
            $localStorage.GetMapDataEx.mapData = data;
            $localStorage.GetMapDataEx.BikesOutOfLocation = false;

            self.distance();

            if (typeof plugin != 'undefined') $localStorage.GetMapDataEx.stations = new plugin.google.maps.LatLng(data.DefaultLatitude, data.DefaultLongitude);
            $localStorage.GetMapDataEx.mapData = data;

            if (AppConfig.BackendSettings.RENTAL_ALLOW_EXTERNAL_RETURN_ANYWHERE == "1" || AppConfig.BackendSettings.RENTAL_ALLOW_EXTERNAL_RENTAL_ANYWHERE == "1") {
                GetBikesOutOfLocation.query(function (data) {
                    toUpdate--;
                    $localStorage.GetMapDataEx.BikesOutOfLocation = data;
                    if (toUpdate === 0) {
                        if (callback !== false) {
                            callback();
                        }
                    }
                });
            }

            angular.forEach($localStorage.GetMapDataEx.mapData.Locations, function (location) {
                location.Status = {
                    bikes: [],
                    BikeAvailabilityTypes: {},
                    locksDetail: {
                        EBike: 0,
                        Classic: 0
                    },
                    locks: 0,
                    freeLocks: 0,
                    totalBikes: 0
                };

                angular.forEach(ShowBikeAvailabilityTypes, function (type) {
                    location.Status.BikeAvailabilityTypes[type] = 0;
                });

                var TotalAvailabilityBikes = 0;
                angular.forEach(location.Stations, function (station) {
                    if (station.IsEBikeStation) {
                        location.IsEBikeStation = true;
                        location.Status.locksDetail.EBike += station.LockCount;
                    } else {
                        location.Status.locksDetail.Classic += station.LockCount;
                    }

                    location.Status.locks += station.LockCount;

                    angular.forEach(ShowBikeAvailabilityTypes, function (type) {
                        angular.forEach(station[type], function (item) {
                            if (type === "AvailableBikes") {
                                location.Status.bikes.push(item);
                            } else {
                                TotalAvailabilityBikes++;
                                location.Status.BikeAvailabilityTypes[type]++;
                            }
                        });
                    });
                });

                location.Status.freeLocks = location.Status.locks - location.Status.bikes.length - TotalAvailabilityBikes;
                location.Status.totalBikes = location.Status.bikes.length + location.AvailableExternalBikes.length;
                console.log(location);
            });

            $rootScope.$broadcast('GetMapDataExUpdate');

            toUpdate--;
            if (toUpdate === 0) {
                if (callback !== false) {
                    callback();
                }
            }
        }, function () {
            // zkusit znovu po 5 vterinach
            $timeout(function () {
                self.get(callback);
            }, 5000)
        });
    }
});

HS.factory('EBikeMotorProfiles', function ($resource, $localStorage, AppConfig) {
    return $resource(AppConfig.backend.url + "EBikeMotorProfiles", {}, {
        query: {
            isArray: true,
            params: {
                lang: $localStorage.lang,
            }
        }
    });
});

HS.service('ServerLogin', function ($http, FrontendService, $ionicLoading, $translate, $state, $localStorage, $translate, Toast, AppConfig, Error, $rootScope, ServerConnection, Timer, GetCustomerContractsService) {

    var self = this;

    this.processLogin = function () {
        FrontendService.auth();
        $state.go($localStorage.landingPage);

        // nacist TKCustomer
        ServerConnection.GetCustomerServiceGet(function (data) {
            $localStorage.TKCustomer = data;
        }, function (err) {
            Error.solve(err);
        });

        ServerConnection.GetCustomerRentalsServiceOngoing();
        ServerConnection.GetCustomerRentalsServiceClosed();
        ServerConnection.GetCustomerContractsServiceLoad();

        Timer.add("GetCustomerRentalsServiceOngoing", false, function ($scope) {
            ServerConnection.GetCustomerRentalsServiceOngoing();
        }, 120000, false);

        $ionicLoading.hide();
    }

    this.doLogin = function (data) {
        delete $http.defaults.headers.common['Authorization'];
        $ionicLoading.show({
            template: $translate("pages.login.messages.loggingIn").then(function (string) {
                return string;
            })
        });

        if (!data.hasOwnProperty("CardNumber")) throw "Missing CardNumber";
        if (!data.hasOwnProperty("pin")) throw "Missing pin";

        var params = {};
        params.username = data.CardNumber;
        params.password = data.pin;
        params.grant_type = "password";

        params.client_id = AppConfig.backend[$rootScope.platform].client_id;
        params.client_secret = AppConfig.backend[$rootScope.platform].client_secret;

        var request = {
            method: "POST",
            url: AppConfig.backend.url + "Token",
            headers: {
                'Content-Type': 'application/x-www-form-urlencoded'
            },
            data: HS.serialize(params),
            timeout: 10000
        };

        var login = {
            CardNumber: data.CardNumber,
            pin: data.pin
        };

        $http(request).then(function (data) {
            $localStorage.Login = login;
            $localStorage.OAuth = data.data;
            $localStorage.anonymous = false;

            $http.defaults.headers.common['Authorization'] = 'Bearer ' + $localStorage.OAuth.access_token;
            self.processLogin();
        }, function (err) {
            Error.solve(err);
        }).catch(function (err) {
            if (err.data !== null) {
                var message = $translate("pages.login.messages." + err.data.error).then(function (string) {
                    return string;
                });
            } else {
                var message = $translate("error.status.0").then(function (string) {
                    return string;
                });
            }
            Toast.show(message);
        });
    }
});

HS.factory('Toast', function ($sce, $rootScope, $timeout, $ionicPopup, $cordovaToast, AppConfig) {
    return {
        show: function (message, duration, position) {
            if (!message || message.length == 0) return;

            duration = duration || 'short';
            position = position || 'bottom';

            message = parseHtmlEnteties(message);

            if (!!window.cordova) {
                $cordovaToast.show(message, duration, position);
            } else {
                if (duration == 'short') {
                    duration = 2000;
                } else {
                    duration = 5000;
                }

                var myPopup = $ionicPopup.show({
                    template: "<div class='toast'>" + message + "</div>",
                    scope: $rootScope,
                    buttons: []
                });

                $timeout(function () {
                    myPopup.close();
                }, duration);
            }
        }
    };
})